@@ -22,8 +22,29 @@ pub struct oid {
     bytes: [u8],
 }
 
-impl std::fmt::Debug for oid {
-    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
+/// A utility able to format itself with the given amount of characters in hex
+#[derive(PartialEq, Eq, Hash, Ord, PartialOrd)]
+pub struct HexDisplay<'a> {
+    inner: &'a oid,
+    hex_len: usize,
+}
+
+impl<'a> fmt::Display for HexDisplay<'a> {
+    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
+        match self.inner.kind() {
+            crate::Kind::Sha1 => {
+                let buf = self.inner.to_sha1_hex();
+                f.write_str(
+                    std::str::from_utf8(&buf[..self.hex_len.min(crate::Kind::Sha1.len_in_hex())])
+                        .expect("hex is always utf8 representable"),
+                )
+            }
+        }
+    }
+}
+
+impl fmt::Debug for oid {
+    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> std::fmt::Result {
         match self.kind() {
             crate::Kind::Sha1 => f.write_str("Sha1(")?,
         }
@@ -89,6 +110,14 @@ impl oid {
     pub fn as_bytes(&self) -> &[u8] {
         &self.bytes
     }
+
+    /// Return a type which can display itself in hexadecimal form with the `len` amount of characters.
+    pub fn to_hex(&self, len: usize) -> HexDisplay<'_> {
+        HexDisplay {
+            inner: self,
+            hex_len: len,
+        }
+    }
 }
 
 /// Sha1 specific methods
